import sys, os
import struct

prgname = sys.argv[0]
flags = [arg for arg in sys.argv if arg[0] == '-']
files = [arg for arg in sys.argv[1:] if arg[0] != '-']

if '-h' in flags or '--help' in flags or not len(files):
    print('Usage: %s [-b] [-i] files...' % prgname)
    print('    -b    Use big endian (Lumines LIVE!)')
    print('    -i    Ignore input length errors')
    sys.exit(0)

use_be = '-b' in flags
ignore_len = '-i' in flags
if use_be:
    header_fmt = '>IIBxxx'
else:
    header_fmt = '<IIBxxx'

def decode_lumines(fn, infile, outfile):
    header = infile.read(12)
    outsize, insize, flag = struct.unpack(header_fmt, header)
    buf = infile.read()
    outbuf = b''
    if len(buf) != insize:
        msg = "input length mismatch (expected %d, got %d)" % (insize, len(buf))
        if ignore_len:
            print('Warning while decompressing %s: %s' % (fn, msg))
        else:
            raise Exception(msg)
    pos = 0
    escapes = 0
    while pos < len(buf):
        b = buf[pos]
        dec = bytes([b])
        if b != flag:
            # unmodified data, copy
            pos += 1
        elif buf[pos + 1] == flag:
            # escape sequence
            escapes += 1
            pos += 2
        else:
            # backreference
            offset = buf[pos + 1]
            copy = buf[pos + 2]
            rep = copy // offset + 1
            src = outbuf[len(outbuf) - offset:len(outbuf) - offset + min(offset, copy)]
            dec = (src * rep)[:copy]
            pos += 3
        outbuf += dec
        outfile.write(dec)
    if len(outbuf) != outsize:
        raise Exception("output length mismatch (expected %d, got %d)" % (outsize, len(outbuf)))

for fn in files:
    with open('%s.out' % fn, 'wb') as outfile:
        with open(fn, 'rb') as infile:
            try:
                decode_lumines(fn, infile, outfile)
            except Exception as e:
                print('Error while decompressing %s: %s' % (fn, e))
